@@ -40,11 +40,11 @@
 //
 //M*/
 
+#include "opencv2/core/core_c.h"
+#include "opencv2/core/types.hpp"
 #include "precomp.hpp"
 #include "distortion_model.hpp"
 
-#include "calib3d_c_api.h"
-
 #include "undistort.simd.hpp"
 #include "undistort.simd_declarations.hpp" // defines CV_CPU_DISPATCH_MODES_ALL=AVX2,...,BASELINE based on CMakeLists.txt content
 
@@ -333,54 +333,6 @@ void undistort( InputArray _src, OutputArray _dst, InputArray _cameraMatrix,
 
 }
 
-CV_IMPL void
-cvUndistort2( const CvArr* srcarr, CvArr* dstarr, const CvMat* Aarr, const CvMat* dist_coeffs, const CvMat* newAarr )
-{
-    cv::Mat src = cv::cvarrToMat(srcarr), dst = cv::cvarrToMat(dstarr), dst0 = dst;
-    cv::Mat A = cv::cvarrToMat(Aarr), distCoeffs = cv::cvarrToMat(dist_coeffs), newA;
-    if( newAarr )
-        newA = cv::cvarrToMat(newAarr);
-
-    CV_Assert( src.size() == dst.size() && src.type() == dst.type() );
-    cv::undistort( src, dst, A, distCoeffs, newA );
-}
-
-
-CV_IMPL void cvInitUndistortMap( const CvMat* Aarr, const CvMat* dist_coeffs,
-                                 CvArr* mapxarr, CvArr* mapyarr )
-{
-    cv::Mat A = cv::cvarrToMat(Aarr), distCoeffs = cv::cvarrToMat(dist_coeffs);
-    cv::Mat mapx = cv::cvarrToMat(mapxarr), mapy, mapx0 = mapx, mapy0;
-
-    if( mapyarr )
-        mapy0 = mapy = cv::cvarrToMat(mapyarr);
-
-    cv::initUndistortRectifyMap( A, distCoeffs, cv::Mat(), A,
-                                 mapx.size(), mapx.type(), mapx, mapy );
-    CV_Assert( mapx0.data == mapx.data && mapy0.data == mapy.data );
-}
-
-void
-cvInitUndistortRectifyMap( const CvMat* Aarr, const CvMat* dist_coeffs,
-    const CvMat *Rarr, const CvMat* ArArr, CvArr* mapxarr, CvArr* mapyarr )
-{
-    cv::Mat A = cv::cvarrToMat(Aarr), distCoeffs, R, Ar;
-    cv::Mat mapx = cv::cvarrToMat(mapxarr), mapy, mapx0 = mapx, mapy0;
-
-    if( mapyarr )
-        mapy0 = mapy = cv::cvarrToMat(mapyarr);
-
-    if( dist_coeffs )
-        distCoeffs = cv::cvarrToMat(dist_coeffs);
-    if( Rarr )
-        R = cv::cvarrToMat(Rarr);
-    if( ArArr )
-        Ar = cv::cvarrToMat(ArArr);
-
-    cv::initUndistortRectifyMap( A, distCoeffs, R, Ar, mapx.size(), mapx.type(), mapx, mapy );
-    CV_Assert( mapx0.data == mapx.data && mapy0.data == mapy.data );
-}
-
 static void cvUndistortPointsInternal( const CvMat* _src, CvMat* _dst, const CvMat* _cameraMatrix,
                    const CvMat* _distCoeffs,
                    const CvMat* matR, const CvMat* matP, cv::TermCriteria criteria)
@@ -473,19 +425,31 @@ static void cvUndistortPointsInternal( const CvMat* _src, CvMat* _dst, const CvM
             x = srcd[i*sstep].x;
             y = srcd[i*sstep].y;
         }
+        // [u, v]^T = [fx * x''' + cx, fy * y''' + cy]^T =>
+        // [x''', y''']^T = [(u - cx) / fx, (v - cy) / fy]^T
         u = x; v = y;
         x = (x - cx)*ifx;
         y = (y - cy)*ify;
 
         if( _distCoeffs ) {
             // compensate tilt distortion
+            // s * [x''', y''', 1]^T = matTilt * [x'', y'', 1]^T =>
+            // s * matTilt^{-1} * [x''', y''', 1]^T = [x'', y'', 1]^T =>
+            // (invMatTilt := matTilt^{-1}, vecUntilt := invMatTilt * [x''', y''', 1]^T)
+            // s * vecUntilt = [x'', y'', 1]^T =>
+            // s * vecUntilt_1 = x'', s * vecUntilt_2 = y'', s * vecUntilt_3 = 1 =>
+            // invProj := s = 1 / vecUntilt_3, x'' = invProj * vecUntilt_1, y'' = invProj * vecUntilt_2
             cv::Vec3d vecUntilt = invMatTilt * cv::Vec3d(x, y, 1);
             double invProj = vecUntilt(2) ? 1./vecUntilt(2) : 1;
             x0 = x = invProj * vecUntilt(0);
             y0 = y = invProj * vecUntilt(1);
 
             double error = std::numeric_limits<double>::max();
-            // compensate distortion iteratively
+            double prevError = std::numeric_limits<double>::max();
+            // compensate distortion iteratively using fixed-point iteration
+
+            // parameter for damped fixed-point iteration
+            double alpha = 1.;
 
             for( int j = 0; ; j++ )
             {
@@ -493,7 +457,9 @@ static void cvUndistortPointsInternal( const CvMat* _src, CvMat* _dst, const CvM
                     break;
                 if ((criteria.type & cv::TermCriteria::EPS) && error < criteria.epsilon)
                     break;
+                // r^2 = x'^2 + y'^2
                 double r2 = x*x + y*y;
+                // icdist := (1 + k4 * r^2 + k5 * r^4 + k6 * r^6) / (1 + k1 * r^2 + k2 * r^4 + k3 * r^6)
                 double icdist = (1 + ((k[7]*r2 + k[6])*r2 + k[5])*r2)/(1 + ((k[4]*r2 + k[1])*r2 + k[0])*r2);
                 if (icdist < 0)  // test: undistortPoints.regression_14583
                 {
@@ -501,10 +467,18 @@ static void cvUndistortPointsInternal( const CvMat* _src, CvMat* _dst, const CvM
                     y = (v - cy)*ify;
                     break;
                 }
+                // deltaX := 2 * p1 * x' * y' + p2 * (r^2 + 2 * x'^2) + s1 * r^2 + s2 * r^4
+                // deltaY := p1 * (r^2 + 2 * y'^2) + 2 * p2 * x' * y' + s3 * r^2 + s4 * r^4
                 double deltaX = 2*k[2]*x*y + k[3]*(r2 + 2*x*x)+ k[8]*r2+k[9]*r2*r2;
                 double deltaY = k[2]*(r2 + 2*y*y) + 2*k[3]*x*y+ k[10]*r2+k[11]*r2*r2;
-                x = (x0 - deltaX)*icdist;
-                y = (y0 - deltaY)*icdist;
+                // [x'', y'']^T = [x' / icdist + deltaX, y' / icdist + deltaY]^T =>
+                // [x', y']^T = [(x'' - deltaX) * icdist, (y'' - deltaY) * icdist]^T =>
+                // x' = f1(x') := (x'' - deltaX) * icdist, y' = f2(y') := (y'' - deltaY) * icdist
+                // Damped fixed-point iteration:
+                //   f1(x') = (x'' - deltaX) * icdist, f2(y') = (y'' - deltaY) * icdist
+                //   new_x' = (1 - alpha) * x' + alpha * f1(x'), new_y' = (1 - alpha) * y' + alpha * f2(y')
+                double new_x = (1. - alpha)*x + alpha*(x0 - deltaX)*icdist;
+                double new_y = (1. - alpha)*y + alpha*(y0 - deltaY)*icdist;
 
                 if(criteria.type & cv::TermCriteria::EPS)
                 {
@@ -512,35 +486,56 @@ static void cvUndistortPointsInternal( const CvMat* _src, CvMat* _dst, const CvM
                     double xd, yd, xd0, yd0;
                     cv::Vec3d vecTilt;
 
-                    r2 = x*x + y*y;
+                    // r^2 = x'^2 + y'^2
+                    r2 = new_x*new_x + new_y*new_y;
                     r4 = r2*r2;
                     r6 = r4*r2;
-                    a1 = 2*x*y;
-                    a2 = r2 + 2*x*x;
-                    a3 = r2 + 2*y*y;
+                    a1 = 2*new_x*new_y;
+                    a2 = r2 + 2*new_x*new_x;
+                    a3 = r2 + 2*new_y*new_y;
+                    // cdist := 1 + k1 * r^2 + k2 * r^4 + k3 * r^6
                     cdist = 1 + k[0]*r2 + k[1]*r4 + k[4]*r6;
+                    // icdist2 := 1 / (1 + k4 * r^2 + k5 * r^4 + k6 * r^6)
                     icdist2 = 1./(1 + k[5]*r2 + k[6]*r4 + k[7]*r6);
-                    xd0 = x*cdist*icdist2 + k[2]*a1 + k[3]*a2 + k[8]*r2+k[9]*r4;
-                    yd0 = y*cdist*icdist2 + k[2]*a3 + k[3]*a1 + k[10]*r2+k[11]*r4;
+                    // x'' = x' * cdist * icdist2 + 2 * p1 * x' * y' + p2 * (r^2 + 2 * x'^2) + s1 * r^2 + s2 * r^4
+                    // y'' = y' * cdist * icdist2 + p1 * (r^2 + 2 * y'^2) + 2 * p2 * x' * y' + s3 * r^2 + s4 * r^4
+                    xd0 = new_x*cdist*icdist2 + k[2]*a1 + k[3]*a2 + k[8]*r2+k[9]*r4;
+                    yd0 = new_y*cdist*icdist2 + k[2]*a3 + k[3]*a1 + k[10]*r2+k[11]*r4;
 
+                    // s * [x''', y''', 1]^T = matTilt * [x'', y'', 1]^T =>
+                    // (vecTilt := matTilt * [x'', y'', 1]^T)
+                    // s * [x''', y''', 1]^T = vecTilt =>
+                    // s * x''' = vecTilt_1, s * y''' = vecTilt_2, s = vecTilt_3 =>
+                    // invProj := 1 / s = 1 / vecTilt_3, x''' = invProj * vecTilt_1, y''' = invProj * vecTilt_2
                     vecTilt = matTilt*cv::Vec3d(xd0, yd0, 1);
                     invProj = vecTilt(2) ? 1./vecTilt(2) : 1;
                     xd = invProj * vecTilt(0);
                     yd = invProj * vecTilt(1);
 
+                    // [u, v]^T = [fx * x''' + cx, fy * y''' + cy]^T
                     double x_proj = xd*fx + cx;
                     double y_proj = yd*fy + cy;
 
                     error = sqrt( pow(x_proj - u, 2) + pow(y_proj - v, 2) );
                 }
+                if (error > prevError) {
+                    alpha *= .5;
+                } else {
+                    x = new_x;
+                    y = new_y;
+                }
+                prevError = error;
             }
         }
 
-        double xx = RR[0][0]*x + RR[0][1]*y + RR[0][2];
-        double yy = RR[1][0]*x + RR[1][1]*y + RR[1][2];
-        double ww = 1./(RR[2][0]*x + RR[2][1]*y + RR[2][2]);
-        x = xx*ww;
-        y = yy*ww;
+        if( matR || matP )
+        {
+            double xx = RR[0][0]*x + RR[0][1]*y + RR[0][2];
+            double yy = RR[1][0]*x + RR[1][1]*y + RR[1][2];
+            double ww = 1./(RR[2][0]*x + RR[2][1]*y + RR[2][2]);
+            x = xx*ww;
+            y = yy*ww;
+        }
 
         if( dtype == CV_32FC2 )
         {
@@ -555,14 +550,6 @@ static void cvUndistortPointsInternal( const CvMat* _src, CvMat* _dst, const CvM
     }
 }
 
-void cvUndistortPoints(const CvMat* _src, CvMat* _dst, const CvMat* _cameraMatrix,
-                       const CvMat* _distCoeffs,
-                       const CvMat* matR, const CvMat* matP)
-{
-    cvUndistortPointsInternal(_src, _dst, _cameraMatrix, _distCoeffs, matR, matP,
-                              cv::TermCriteria(cv::TermCriteria::COUNT, 5, 0.01));
-}
-
 namespace cv {
 
 void undistortPoints(InputArray _src, OutputArray _dst,
@@ -607,6 +594,11 @@ void undistortPoints(InputArray _src, OutputArray _dst,
     cvUndistortPointsInternal(&_csrc, &_cdst, &_ccameraMatrix, pD, pR, pP, criteria);
 }
 
+void undistortImagePoints(InputArray src, OutputArray dst, InputArray cameraMatrix, InputArray distCoeffs, TermCriteria termCriteria)
+{
+    undistortPoints(src, dst, cameraMatrix, distCoeffs, noArray(), cameraMatrix, termCriteria);
+}
+
 static Point2f mapPointSpherical(const Point2f& p, float alpha, Vec4d* J, enum UndistortTypes projType)
 {
     double x = p.x, y = p.y;
