@@ -118,6 +118,16 @@ Subdiv2D::Subdiv2D(Rect rect)
     initDelaunay(rect);
 }
 
+Subdiv2D::Subdiv2D(Rect2f rect)
+{
+    validGeometry = false;
+    freeQEdge = 0;
+    freePoint = 0;
+    recentEdge = 0;
+
+    initDelaunay(rect);
+}
+
 
 Subdiv2D::QuadEdge::QuadEdge()
 {
@@ -255,7 +265,7 @@ int Subdiv2D::newPoint(Point2f pt, bool isvirtual, int firstEdge)
 {
     if( freePoint == 0 )
     {
-        vtx.push_back(Vertex());
+        vtx.emplace_back();
         freePoint = (int)(vtx.size()-1);
     }
     int vidx = freePoint;
@@ -510,8 +520,54 @@ void Subdiv2D::initDelaunay( Rect rect )
     Point2f ppB( rx, ry + big_coord );
     Point2f ppC( rx - big_coord, ry - big_coord );
 
-    vtx.push_back(Vertex());
-    qedges.push_back(QuadEdge());
+    vtx.emplace_back();
+    qedges.emplace_back();
+
+    freeQEdge = 0;
+    freePoint = 0;
+
+    int pA = newPoint(ppA, false);
+    int pB = newPoint(ppB, false);
+    int pC = newPoint(ppC, false);
+
+    int edge_AB = newEdge();
+    int edge_BC = newEdge();
+    int edge_CA = newEdge();
+
+    setEdgePoints( edge_AB, pA, pB );
+    setEdgePoints( edge_BC, pB, pC );
+    setEdgePoints( edge_CA, pC, pA );
+
+    splice( edge_AB, symEdge( edge_CA ));
+    splice( edge_BC, symEdge( edge_AB ));
+    splice( edge_CA, symEdge( edge_BC ));
+
+    recentEdge = edge_AB;
+}
+
+void Subdiv2D::initDelaunay( Rect2f rect )
+{
+        CV_INSTRUMENT_REGION();
+
+    float big_coord = 6.f * MAX( rect.width, rect.height );
+    float rx = rect.x;
+    float ry = rect.y;
+
+    vtx.clear();
+    qedges.clear();
+
+    recentEdge = 0;
+    validGeometry = false;
+
+    topLeft = Point2f( rx, ry );
+    bottomRight = Point2f( rx + rect.width, ry + rect.height );
+
+    Point2f ppA( rx + big_coord, ry );
+    Point2f ppB( rx, ry + big_coord );
+    Point2f ppC( rx - big_coord, ry - big_coord );
+
+    vtx.emplace_back();
+    qedges.emplace_back();
 
     freeQEdge = 0;
     freePoint = 0;
@@ -728,7 +784,7 @@ void Subdiv2D::getEdgeList(std::vector<Vec4f>& edgeList) const
         {
             Point2f org = vtx[qedges[i].pt[0]].pt;
             Point2f dst = vtx[qedges[i].pt[2]].pt;
-            edgeList.push_back(Vec4f(org.x, org.y, dst.x, dst.y));
+            edgeList.emplace_back(org.x, org.y, dst.x, dst.y);
         }
     }
 }
@@ -780,7 +836,7 @@ void Subdiv2D::getTriangleList(std::vector<Vec6f>& triangleList) const
         edgemask[edge_a] = true;
         edgemask[edge_b] = true;
         edgemask[edge_c] = true;
-        triangleList.push_back(Vec6f(a.x, a.y, b.x, b.y, c.x, c.y));
+        triangleList.emplace_back(a.x, a.y, b.x, b.y, c.x, c.y);
     }
 }
 
